import math
from mpython import *

class Heartrate:
  SAMPLE_NUMBER                  = 100
  ANALOG_MODE                    = 0
  DIGITAL_MODE                   = 1
  def __init__(self):
    self.value = [0 for i in range(self.SAMPLE_NUMBER)]
    self.value_count = 255
    self.now_time = 0
    self.last_time = 0
    self.value_flag = 0
    self.mode = self.ANALOG_MODE
    self.tnew = 0
    self.sample_time = [0 for i in range(10)]
    self.time_flag = 0

  def begin(self, pin, mode = ANALOG_MODE):
    self.pin = pin
    self.mode = mode
    self.adc = MPythonPin(pin, PinMode.ANALOG)
  
  def get_value(self):
    self.value_count = self.value_count + 1
    if self.value_count >= self.SAMPLE_NUMBER:
      self.value_count = 0
    self.value[self.value_count] = math.ceil(self.adc.read_analog() / 4)
  
  def min_number(self, count):
    temp1 = 0
    temp2 = 0
    for i in range(4):
      if count < i:
        temp1 = self.SAMPLE_NUMBER + count - i
      else:
        temp1 = count - i
      if count < (i+1):
        temp2 = self.SAMPLE_NUMBER + (count - 1) - i
      else:
        temp2 = (count - 1) - i
      if self.value[temp1] >= self.value[temp2]:
        return 
    self.value_flag = 0
  
  def max_number(self, count):
    for i in range(4):
      if count < i:
        temp1 = self.SAMPLE_NUMBER + count - i
      else:
        temp1 = count - i
      if count < (i+1):
        temp2 = self.SAMPLE_NUMBER + (count - 1) - i
      else:
        temp2 = (count - 1) - i
      if self.value[temp1] <= self.value[temp2]:
        return 0
    if self.value_flag:
      self.value_flag = 1
      return 0
    else:
      self.value_flag = 1
      return 1

  def digital_get_rate(self):
    if self.value_count:
      count = self.value_count - 1
    else:
      count = self.SAMPLE_NUMBER - 1
    if self.value[self.value_count] > 1000 and self.value[count] < 20:
      self.now_time = time.ticks_ms()
      dif_time = self.now_time - self.last_time
      self.last_time = self.now_time
      if dif_time > 300 or dif_time < 2000:
        self.sample_time[self.time_flag] = dif_time
        self.time_flag = self.time_flag + 1
        if self.time_flag > 9:
          self.time_flag = 0
        if 0 == self.sample_time[9]:
#          print("Wait for valid data !")
          return 0
        arrange = [0 for i in range(10)]
        for i in range(10):
          arrange[i] = self.sample_time[i]
        arrange_ = 0
        for i in range(9,-1,-1):
          for j in range(i):
            if arrange[j] > arrange[j+1]:
              arrange_ = arrange[j]
              arrange[j] = arrange[j+1]
              arrange[j+1] = arrange_
        if arrange[7] - arrange[3] > 120:
#          print("Wait for valid data !")
          return 0
        arrange_ = 0
        for i in range(3,8):
          arrange_ += arrange[i]
        value_time = (int)(300000 / arrange_)
        return value_time
    
  def analog_get_rate(self):
    valueTime_ = 0
    self.min_number(self.value_count)
    if self.max_number(self.value_count):
      self.now_time = time.ticks_ms()
      dif_time = self.now_time - self.last_time
      self.last_time = self.now_time
      if dif_time > 300 or dif_time < 2000:
        self.sample_time[self.time_flag] = dif_time
        self.time_flag = self.time_flag + 1
        if self.time_flag > 9:
          self.time_flag = 0
      if 0 == self.sample_time[9]:
        #print("Wait for valid data !")
        return 0
      arrange = [0 for i in range(10)]
      for i in range(10):
        arrange[i] = self.sample_time[i]
      arrange_ = 0
      for i in range(9,-1,-1):
        for j in range(i):
          if arrange[j] > arrange[j+1]:
            arrange_ = arrange[j]
            arrange[j] = arrange[j+1]
            arrange[j+1] = arrange_
      if arrange[7] - arrange[3] > 150:
          #print("Wait for valid data !")
          return 0
      arrange_ = 0
      for i in range(3,8):
        arrange_ += arrange[i]
      value_time = (int)(300000 / arrange_)
      return value_time
    return 0

  def get_rate(self):
    val = 0
    t = 0
    self.num = 1
    self.first = True
    self.told = 0
    self.tnew= time.ticks_ms()
    while not val:
      self.get_value()
      if self.mode == self.DIGITAL_MODE:
        val = self.digital_get_rate()
      else:
        val = self.analog_get_rate()
      if val:
        self.num = 1
        self.told = time.ticks_ms()
        break
      sleep_ms(20)
      if time.ticks_ms() - self.told > 2000 or self.first:
        if self.first:
          self.first = False
        if self.num:
          print("Wait for valid data !")
          self.num = 0
        if t % 50 == 0:
          print(".", end= "")  
        t = t + 1
      if time.ticks_ms() - self.tnew >15000:
        self.num = 1
        print("time out")
        return val
    if t:
      print("OK")
    return val
    